//-----------------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited or its affiliates.
//
//            (C) COPYRIGHT 2012-2020 ARM Limited or its affiliates.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited or its affiliates.
//
//            Release Information : CORTEX-A65AE-MP083-r1p0-00eac0
//
//-----------------------------------------------------------------------------
// SystemVerilog-2012 (IEEE Std 1800-2012)
//-----------------------------------------------------------------------------


`include "helios_dpu_params.sv"
`include "helios_params.sv"

module helios_dpu_crypto (
  input  logic                           clk_crypto,
  input  logic                           reset_n,
  input  logic                           advance_pipeline_i,
  input  logic                           crypto_enable_f1_i,
  input  logic                   [127:0] crypto_a_data_f1_i,
  input  logic                   [127:0] crypto_b_data_f1_i,
  input  logic                   [127:0] crypto_c_data_f1_i,
  input  logic [`HELIOS_CRYPTO_OP_W-1:0] crypto_op_f1_i,
  output logic                           crypto_active_o,
  output logic                   [127:0] crypto_data_f3_o,
  output logic                   [127:0] crypto_data_f5_o
);



  logic                   [127: 0] aes_add_roundkey_f2;
  logic                   [127: 0] aes_fwd_sbox_res_f2;
  logic                   [127: 0] aes_inv_sbox_res_f2;
  logic                            aes_invmixcols_f2;
  logic                            aes_invmixcols_f3;
  logic                   [127: 0] aes_invshiftrows_f2;
  logic                   [127: 0] aes_mixcol_data_f3;
  logic                   [127: 0] aes_shiftrows_f2;
  logic                   [127: 0] crypto_a_data_f2;
  logic                   [127: 0] crypto_c_data_f2;
  logic                   [127: 0] crypto_a_data_f3;
  logic                   [127:64] crypto_a_data_f4;
  logic                   [127:96] crypto_a_data_f5;
  logic                   [127: 0] crypto_b_data_f2;
  logic                   [127: 0] crypto_b_data_f3;
  logic                   [127: 0] crypto_b_data_f4;
  logic                    [95: 0] crypto_b_data_f5;
  logic                   [127: 0] crypto_c_data_f3;
  logic                   [127: 0] crypto_c_data_f4;
  logic                   [127: 0] crypto_c_data_f5;
  logic                            crypto_enable_f2;
  logic                            crypto_enable_f3;
  logic                            crypto_enable_f4;
  logic [`HELIOS_CRYPTO_OP_W-1: 0] crypto_op_f2;
  logic [`HELIOS_CRYPTO_OP_W-1: 0] crypto_op_f3;
  logic [`HELIOS_CRYPTO_OP_W-1: 0] crypto_op_f4;
  logic [`HELIOS_CRYPTO_OP_W-1: 0] crypto_op_f5;
  logic                     [1: 0] en_b_data_f2;
  logic                     [3: 0] en_c_data_f2;
  logic                            en_crypto_op_f2;
  logic                            en_crypto_op_f3;
  logic                            en_crypto_op_f4;
  logic                            en_hi_c_data_f3;
  logic                            en_hi_c_data_f4;
  logic                            en_hi_c_data_f5;
  logic                            en_lo_bc_data_f3;
  logic                   [255: 0] hash_data_f2;
  logic                   [255: 0] hash_data_f3;
  logic                   [255: 0] hash_data_f4;
  logic                   [159: 0] hash_data_sha1_f5;
  logic                   [255: 0] hash_data_sha256_f5;
  logic                   [127: 0] nxt_crypto_a_data_f3;
  logic                   [127: 0] nxt_crypto_b_data_f3;
  logic                   [127: 0] nxt_crypto_b_data_f4;
  logic                    [95: 0] nxt_crypto_b_data_f5;
  logic                   [127: 0] nxt_crypto_c_data_f3;
  logic                   [127: 0] nxt_crypto_c_data_f4;
  logic                   [127: 0] nxt_crypto_c_data_f5;
  logic                            nxt_crypto_enable_f4;
  logic                   [126: 0] pmull64_res_f2;
  logic                            res_sel_a_f2;
  logic                     [1: 0] res_sel_a_f3;
  logic                            res_sel_mc_f2;
  logic                     [1: 0] res_sel_mc_f3;
  logic                   [127: 0] sha1su1_t_f2;
  logic                   [127: 0] sha1su1_res_f2;
  logic                   [159: 0] sha1cpm_res_f2;
  logic                   [159: 0] sha1cpm_res_f3;
  logic                   [159: 0] sha1cpm_res_f4;
  logic                   [159: 0] sha1cpm_res_f5;
  logic                   [255: 0] sha256h_res_f2;
  logic                   [255: 0] sha256h_res_f3;
  logic                   [255: 0] sha256h_res_f4;
  logic                   [255: 0] sha256h_res_f5;
  logic                    [63: 0] sha256su1_res_f2;
  logic                    [63: 0] sha256su1_res_f3;

  logic det_reset_n;

    assign det_reset_n = reset_n;


  always_ff @(posedge clk_crypto or negedge reset_n)
    if (~reset_n)
      crypto_enable_f2 <= '0;
    else if (advance_pipeline_i)
      crypto_enable_f2 <= crypto_enable_f1_i;

  assign en_crypto_op_f2 = crypto_enable_f1_i & advance_pipeline_i;

  always_ff @(posedge clk_crypto or negedge det_reset_n)
    if (~det_reset_n) begin
      crypto_op_f2     <= '0;
      crypto_a_data_f2 <= '0;
    end else if (en_crypto_op_f2) begin
      crypto_op_f2     <= crypto_op_f1_i;
      crypto_a_data_f2 <= crypto_a_data_f1_i;
    end

  assign en_b_data_f2[0] = crypto_enable_f1_i & advance_pipeline_i &
                           ((crypto_op_f1_i == `HELIOS_CRYPTO_OP_AESD)        |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_AESD_AESIMC) |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_AESE)        |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_AESE_AESMC)  |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_PMULL64)     |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA1C)       |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA1M)       |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA1P)       |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256H)     |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256H2)    |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256SU1));

  assign en_b_data_f2[1] = crypto_enable_f1_i & advance_pipeline_i &
                           ((crypto_op_f1_i == `HELIOS_CRYPTO_OP_AESD)        |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_AESD_AESIMC) |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_AESE)        |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_AESE_AESMC)  |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA1C)       |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA1M)       |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA1P)       |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256H)     |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256H2)    |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256SU1));

  generate for (genvar i = 0; i < 2; i++) begin : g_b_data_f2

    always_ff @(posedge clk_crypto or negedge det_reset_n)
      if (~det_reset_n)
        crypto_b_data_f2[(i * 64)+:64] <= '0;
      else if (en_b_data_f2[i])
        crypto_b_data_f2[(i * 64)+:64] <= crypto_b_data_f1_i[(i * 64)+:64];

  end endgenerate

  assign en_c_data_f2[0] = crypto_enable_f1_i & advance_pipeline_i &
                           ((crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA1C)    |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA1M)    |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA1P)    |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256H)  |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256H2) |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256SU1));

  assign en_c_data_f2[1] = crypto_enable_f1_i & advance_pipeline_i &
                           ((crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA1SU1) |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256H) |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256H2));

// WATERMARK START
  assign en_c_data_f2[2] = crypto_enable_f1_i & advance_pipeline_i &
                           ((crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA1SU1)  |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256H)  |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256H2) |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256SU1));
// WATERMARK VARIANTS 1
//  assign en_c_data_f2[2] = crypto_enable_f1_i & advance_pipeline_i &
//                           ((crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA1SU1)  |
//                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256H2) |
//                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256H)  |
//                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256SU1));
// WATERMARK VARIANTS 2
//  assign en_c_data_f2[2] = crypto_enable_f1_i & advance_pipeline_i &
//                           ((crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256H2) |
//                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256H)  |
//                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA1SU1)  |
//                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256SU1));
// WATERMARK VARIANTS 3
//  assign en_c_data_f2[2] = crypto_enable_f1_i & advance_pipeline_i &
//                           ((crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256SU1) |
//                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256H)   |
//                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256H2)  |
//                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA1SU1));
// WATERMARK VARIANTS 4
//  assign en_c_data_f2[2] = crypto_enable_f1_i & advance_pipeline_i &
//                           ((crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA1SU1)   |
//                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256SU1) |
//                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256H2)  |
//                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256H));
// WATERMARK END

  assign en_c_data_f2[3] = crypto_enable_f1_i & advance_pipeline_i &
                           ((crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA1SU1)  |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256H)  |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256H2) |
                            (crypto_op_f1_i == `HELIOS_CRYPTO_OP_SHA256SU1));

  generate for (genvar i = 0; i < 4; i++) begin : g_c_data_f2

    always_ff @(posedge clk_crypto or negedge det_reset_n)
      if (~det_reset_n)
        crypto_c_data_f2[(i * 32)+:32] <= '0;
      else if (en_c_data_f2[i])
        crypto_c_data_f2[(i * 32)+:32] <= crypto_c_data_f1_i[(i * 32)+:32];

  end endgenerate



  assign aes_add_roundkey_f2 = crypto_a_data_f2 ^ crypto_b_data_f2;

  assign hash_data_f2 = {crypto_c_data_f2, crypto_b_data_f2};

  assign sha1su1_t_f2 = crypto_a_data_f2 ^ {{32{1'b0}}, crypto_c_data_f2[127:32]};


  generate for (genvar i = 0; i < 16; i++) begin : g_aes_sbox

    helios_dpu_crypto_aes_sbox u_aes_sbox (
      .crypto_a_data_f2_i     (aes_add_roundkey_f2[i*8+:8]),
      .aes_fwd_sbox_res_f2_o  (aes_fwd_sbox_res_f2[i*8+:8]),
      .aes_inv_sbox_res_f2_o  (aes_inv_sbox_res_f2[i*8+:8])
    );

  end endgenerate

  assign aes_shiftrows_f2    = {aes_fwd_sbox_res_f2[ 95: 88],  
                                aes_fwd_sbox_res_f2[ 55: 48],  
                                aes_fwd_sbox_res_f2[ 15:  8],  
                                aes_fwd_sbox_res_f2[103: 96],  
                                aes_fwd_sbox_res_f2[ 63: 56],  
                                aes_fwd_sbox_res_f2[ 23: 16],  
                                aes_fwd_sbox_res_f2[111:104],  
                                aes_fwd_sbox_res_f2[ 71: 64],  
                                aes_fwd_sbox_res_f2[ 31: 24],  
                                aes_fwd_sbox_res_f2[119:112],  
                                aes_fwd_sbox_res_f2[ 79: 72],  
                                aes_fwd_sbox_res_f2[ 39: 32],  
                                aes_fwd_sbox_res_f2[127:120],  
                                aes_fwd_sbox_res_f2[ 87: 80],  
                                aes_fwd_sbox_res_f2[ 47: 40],  
                                aes_fwd_sbox_res_f2[  7:  0]}; 

  assign aes_invshiftrows_f2 = {aes_inv_sbox_res_f2[ 31: 24],  
                                aes_inv_sbox_res_f2[ 55: 48],  
                                aes_inv_sbox_res_f2[ 79: 72],  
                                aes_inv_sbox_res_f2[103: 96],  
                                aes_inv_sbox_res_f2[127:120],  
                                aes_inv_sbox_res_f2[ 23: 16],  
                                aes_inv_sbox_res_f2[ 47: 40],  
                                aes_inv_sbox_res_f2[ 71: 64],  
                                aes_inv_sbox_res_f2[ 95: 88],  
                                aes_inv_sbox_res_f2[119:112],  
                                aes_inv_sbox_res_f2[ 15:  8],  
                                aes_inv_sbox_res_f2[ 39: 32],  
                                aes_inv_sbox_res_f2[ 63: 56],  
                                aes_inv_sbox_res_f2[ 87: 80],  
                                aes_inv_sbox_res_f2[111:104],  
                                aes_inv_sbox_res_f2[  7:  0]}; 

  helios_dpu_crypto_sha1cpm u_sha1cpm_f2 (
    .crypto_op_i      (crypto_op_f2),
    .hash_data_i      (hash_data_f2[159:0]),
    .schedule_data_i  (crypto_a_data_f2[31:0]),
    .sha1cpm_res_o    (sha1cpm_res_f2)
  );

  helios_dpu_crypto_sha256h u_sha256h_f2 (
    .hash_data_i      (hash_data_f2),
    .schedule_data_i  (crypto_a_data_f2[31:0]),
    .sha256h_res_o    (sha256h_res_f2)
  );

  helios_dpu_crypto_sha256su1 u_sha256su1_f2 (
    .op1_data_i       (crypto_b_data_f2[63:0]),
    .t0_data_i        (crypto_a_data_f2[95:32]),
    .t1_data_i        (crypto_c_data_f2[127:64]),
    .sha256su1_res_o  (sha256su1_res_f2[63:0])
  );

  helios_dpu_crypto_pmull64 u_pmull64 (
    .opa_i          (crypto_a_data_f2[63:0]),
    .opb_i          (crypto_b_data_f2[63:0]),
    .pmull64_res_o  (pmull64_res_f2[126:0])
  );


  assign sha1su1_res_f2[ 31: 0] = {sha1su1_t_f2[ 30: 0], sha1su1_t_f2[ 31]};
  assign sha1su1_res_f2[ 63:32] = {sha1su1_t_f2[ 62:32], sha1su1_t_f2[ 63]};
  assign sha1su1_res_f2[ 95:64] = {sha1su1_t_f2[ 94:64], sha1su1_t_f2[ 95]};
  assign sha1su1_res_f2[127:96] = {sha1su1_t_f2[126:96], sha1su1_t_f2[127]} ^ {sha1su1_t_f2[29: 0], sha1su1_t_f2[31:30]};

  assign aes_invmixcols_f2 = (crypto_op_f2 == `HELIOS_CRYPTO_OP_AESIMC) |
                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_AESD_AESIMC);

  assign res_sel_a_f2  = (crypto_op_f2 == `HELIOS_CRYPTO_OP_AESD)    |
                         (crypto_op_f2 == `HELIOS_CRYPTO_OP_AESE)    |
                         (crypto_op_f2 == `HELIOS_CRYPTO_OP_PMULL64) |
                         (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA1SU1);

  assign res_sel_mc_f2 = (crypto_op_f2 == `HELIOS_CRYPTO_OP_AESMC)       |
                         (crypto_op_f2 == `HELIOS_CRYPTO_OP_AESIMC)      |
                         (crypto_op_f2 == `HELIOS_CRYPTO_OP_AESD_AESIMC) |
                         (crypto_op_f2 == `HELIOS_CRYPTO_OP_AESE_AESMC);


  always_comb
    case (crypto_op_f2)
      `HELIOS_CRYPTO_OP_SHA1C,
      `HELIOS_CRYPTO_OP_SHA1P,
      `HELIOS_CRYPTO_OP_SHA1M,
      `HELIOS_CRYPTO_OP_SHA256H,
      `HELIOS_CRYPTO_OP_SHA256H2,
      `HELIOS_CRYPTO_OP_AESMC,
      `HELIOS_CRYPTO_OP_AESIMC:
        nxt_crypto_a_data_f3 = crypto_a_data_f2;

      `HELIOS_CRYPTO_OP_SHA256SU1:
        nxt_crypto_a_data_f3 = {{64{1'b0}}, sha256su1_res_f2};

      `HELIOS_CRYPTO_OP_AESE,
      `HELIOS_CRYPTO_OP_AESE_AESMC:
        nxt_crypto_a_data_f3 = aes_shiftrows_f2;

      `HELIOS_CRYPTO_OP_AESD,
      `HELIOS_CRYPTO_OP_AESD_AESIMC:
        nxt_crypto_a_data_f3 = aes_invshiftrows_f2;

      `HELIOS_CRYPTO_OP_PMULL64:
        nxt_crypto_a_data_f3 = {1'b0, pmull64_res_f2};

      `HELIOS_CRYPTO_OP_SHA1SU1:
        nxt_crypto_a_data_f3 = sha1su1_res_f2;

      default:
        nxt_crypto_a_data_f3 = '0;
    endcase

  always_comb
    case (crypto_op_f2)
      `HELIOS_CRYPTO_OP_SHA1C,
      `HELIOS_CRYPTO_OP_SHA1P,
      `HELIOS_CRYPTO_OP_SHA1M: begin
        nxt_crypto_b_data_f3 = sha1cpm_res_f2[127:0];
        nxt_crypto_c_data_f3 = {{96{1'b0}}, sha1cpm_res_f2[159:128]};
      end

      `HELIOS_CRYPTO_OP_SHA256H,
      `HELIOS_CRYPTO_OP_SHA256H2: begin
        nxt_crypto_b_data_f3 = sha256h_res_f2[127:  0];
        nxt_crypto_c_data_f3 = sha256h_res_f2[255:128];
      end

      `HELIOS_CRYPTO_OP_SHA256SU1: begin
        nxt_crypto_b_data_f3 = crypto_b_data_f2;
        nxt_crypto_c_data_f3 = {crypto_c_data_f2[31:0], crypto_a_data_f2[127:32]};
      end

      default: begin
        nxt_crypto_b_data_f3 = '0;
        nxt_crypto_c_data_f3 = '0;
      end
    endcase

// WATERMARK START
  assign en_lo_bc_data_f3 = crypto_enable_f2 & advance_pipeline_i &
                            ((crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA1C)    |
                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA1P)    |
                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA1M)    |
                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA256H)  |
                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA256H2) |
                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA256SU1));
// WATERMARK VARIANTS 1
//  assign en_lo_bc_data_f3 = crypto_enable_f2 & advance_pipeline_i &
//                            ((crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA1P)    |
//                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA1C)    |
//                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA1M)    |
//                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA256H)  |
//                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA256H2) |
//                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA256SU1));
// WATERMARK VARIANTS 2
//  assign en_lo_bc_data_f3 = crypto_enable_f2 & advance_pipeline_i &
//                            ((crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA1P)    |
//                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA1M)    |
//                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA1C)    |
//                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA256H)  |
//                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA256H2) |
//                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA256SU1));
// WATERMARK VARIANTS 3
//  assign en_lo_bc_data_f3 = crypto_enable_f2 & advance_pipeline_i &
//                            ((crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA1M)    |
//                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA1C)    |
//                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA1P)    |
//                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA256H)  |
//                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA256H2) |
//                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA256SU1));
// WATERMARK VARIANTS 4
//  assign en_lo_bc_data_f3 = crypto_enable_f2 & advance_pipeline_i &
//                            ((crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA1M)    |
//                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA1P)    |
//                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA1C)    |
//                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA256H)  |
//                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA256H2) |
//                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA256SU1));
// WATERMARK END

  assign en_hi_c_data_f3  = en_lo_bc_data_f3 &
                            ((crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA256H)   |
                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA256H2)  |
                             (crypto_op_f2 == `HELIOS_CRYPTO_OP_SHA256SU1));

  always_ff @(posedge clk_crypto or negedge reset_n)
    if (~reset_n)
      crypto_enable_f3 <= '0;
    else if (advance_pipeline_i)
      crypto_enable_f3 <= crypto_enable_f2;

  assign en_crypto_op_f3 = crypto_enable_f2 & advance_pipeline_i;

  always_ff @(posedge clk_crypto or negedge det_reset_n)
    if (~det_reset_n) begin
      aes_invmixcols_f3 <= '0;
      crypto_a_data_f3  <= '0;
      crypto_op_f3      <= '0;
    end else if (en_crypto_op_f3) begin
      aes_invmixcols_f3 <= aes_invmixcols_f2;
      crypto_a_data_f3  <= nxt_crypto_a_data_f3;
      crypto_op_f3      <= crypto_op_f2;
    end

  always_ff @(posedge clk_crypto or negedge reset_n)
    if (~reset_n) begin
      res_sel_a_f3[0]  <= '0;
      res_sel_a_f3[1]  <= '1;
      res_sel_mc_f3[0] <= '0;
      res_sel_mc_f3[1] <= '1;
    end else if (en_crypto_op_f3) begin
      res_sel_a_f3[0]  <= res_sel_a_f2;
      res_sel_a_f3[1]  <= res_sel_a_f2;
      res_sel_mc_f3[0] <= res_sel_mc_f2;
      res_sel_mc_f3[1] <= res_sel_mc_f2;
    end

  always_ff @(posedge clk_crypto or negedge det_reset_n)
    if (~det_reset_n) begin
      crypto_b_data_f3       <= '0;
      crypto_c_data_f3[31:0] <= '0;
    end else if (en_lo_bc_data_f3) begin
      crypto_b_data_f3       <= nxt_crypto_b_data_f3;
      crypto_c_data_f3[31:0] <= nxt_crypto_c_data_f3[31:0];
    end

  always_ff @(posedge clk_crypto or negedge det_reset_n)
    if (~det_reset_n)
      crypto_c_data_f3[127:32] <= '0;
    else if (en_hi_c_data_f3)
      crypto_c_data_f3[127:32] <= nxt_crypto_c_data_f3[127:32];



  helios_dpu_crypto_aes_mixcol u_aes_mixcol_0 (
    .aes_invmixcols_f3_i  (aes_invmixcols_f3),
    .crypto_a_data_f3_i   (crypto_a_data_f3[31: 0]),
    .aes_mixcol_data_f3_o (aes_mixcol_data_f3[31: 0])
  );

  helios_dpu_crypto_aes_mixcol u_aes_mixcol_1 (
    .aes_invmixcols_f3_i  (aes_invmixcols_f3),
    .crypto_a_data_f3_i   (crypto_a_data_f3[63:32]),
    .aes_mixcol_data_f3_o (aes_mixcol_data_f3[63:32])
  );

  helios_dpu_crypto_aes_mixcol u_aes_mixcol_2 (
    .aes_invmixcols_f3_i  (aes_invmixcols_f3),
    .crypto_a_data_f3_i   (crypto_a_data_f3[95:64]),
    .aes_mixcol_data_f3_o (aes_mixcol_data_f3[95:64])
  );

  helios_dpu_crypto_aes_mixcol u_aes_mixcol_3 (
    .aes_invmixcols_f3_i  (aes_invmixcols_f3),
    .crypto_a_data_f3_i   (crypto_a_data_f3[127:96]),
    .aes_mixcol_data_f3_o (aes_mixcol_data_f3[127:96])
  );

  assign hash_data_f3 = {crypto_c_data_f3, crypto_b_data_f3};

  helios_dpu_crypto_sha1cpm u_sha1cpm_f3 (
    .crypto_op_i      (crypto_op_f3),
    .hash_data_i      (hash_data_f3[159:0]),
    .schedule_data_i  (crypto_a_data_f3[63:32]),
    .sha1cpm_res_o    (sha1cpm_res_f3)
  );

  helios_dpu_crypto_sha256h u_sha256h_f3 (
    .hash_data_i      (hash_data_f3),
    .schedule_data_i  (crypto_a_data_f3[63:32]),
    .sha256h_res_o    (sha256h_res_f3)
  );

  helios_dpu_crypto_sha256su1 u_sha256su1_f3 (
    .op1_data_i       (crypto_b_data_f3[127:64]),
    .t0_data_i        (crypto_c_data_f3[127:64]),
    .t1_data_i        (crypto_a_data_f3[63:0]),
    .sha256su1_res_o  (sha256su1_res_f3[63:0])
  );


  assign crypto_data_f3_o[63:0]   = ({64{res_sel_a_f3[0]}}  & crypto_a_data_f3[63:0]) |
                                    ({64{res_sel_mc_f3[0]}} & aes_mixcol_data_f3[63:0]);

  assign crypto_data_f3_o[127:64] = ({64{res_sel_a_f3[1]}}  & crypto_a_data_f3[127:64]) |
                                    ({64{res_sel_mc_f3[1]}} & aes_mixcol_data_f3[127:64]);

  always_comb
    case (crypto_op_f3)
      `HELIOS_CRYPTO_OP_SHA1C,
      `HELIOS_CRYPTO_OP_SHA1P,
      `HELIOS_CRYPTO_OP_SHA1M: begin
        nxt_crypto_b_data_f4 = sha1cpm_res_f3[127:0];
        nxt_crypto_c_data_f4 = {{96{1'b0}}, sha1cpm_res_f3[159:128]};
      end

      `HELIOS_CRYPTO_OP_SHA256H,
      `HELIOS_CRYPTO_OP_SHA256H2: begin
        nxt_crypto_b_data_f4 = sha256h_res_f3[127:0];
        nxt_crypto_c_data_f4 = sha256h_res_f3[255:128];
      end

      `HELIOS_CRYPTO_OP_SHA256SU1: begin
        nxt_crypto_b_data_f4 = {sha256su1_res_f3[63:0], crypto_a_data_f3[63:0]};
        nxt_crypto_c_data_f4 = '0;
      end

      default: begin
        nxt_crypto_b_data_f4 = '0;
        nxt_crypto_c_data_f4 = '0;
      end
    endcase

  assign nxt_crypto_enable_f4 = crypto_enable_f3 & advance_pipeline_i &
                                ((crypto_op_f3 == `HELIOS_CRYPTO_OP_SHA1C)    |
                                 (crypto_op_f3 == `HELIOS_CRYPTO_OP_SHA1P)    |
                                 (crypto_op_f3 == `HELIOS_CRYPTO_OP_SHA1M)    |
                                 (crypto_op_f3 == `HELIOS_CRYPTO_OP_SHA256H)  |
                                 (crypto_op_f3 == `HELIOS_CRYPTO_OP_SHA256H2) |
                                 (crypto_op_f3 == `HELIOS_CRYPTO_OP_SHA256SU1));

  assign en_hi_c_data_f4      = nxt_crypto_enable_f4 &
                                ((crypto_op_f3 == `HELIOS_CRYPTO_OP_SHA256H)  |
                                 (crypto_op_f3 == `HELIOS_CRYPTO_OP_SHA256H2));

  always_ff @(posedge clk_crypto or negedge reset_n)
    if (~reset_n)
      crypto_enable_f4 <= '0;
    else if (advance_pipeline_i)
      crypto_enable_f4 <= nxt_crypto_enable_f4;

  always_ff @(posedge clk_crypto or negedge det_reset_n)
    if (~det_reset_n) begin
      crypto_a_data_f4[127:64] <= '0;
      crypto_b_data_f4         <= '0;
      crypto_c_data_f4[31:0]   <= '0;
      crypto_op_f4             <= '0;
    end else if (nxt_crypto_enable_f4) begin
      crypto_a_data_f4[127:64] <= crypto_a_data_f3[127:64];
      crypto_b_data_f4         <= nxt_crypto_b_data_f4;
      crypto_c_data_f4[31:0]   <= nxt_crypto_c_data_f4[31:0];
      crypto_op_f4             <= crypto_op_f3;
    end

  always_ff @(posedge clk_crypto  or negedge det_reset_n)
    if (~det_reset_n)
      crypto_c_data_f4[127:32] <= '0;
    else if (en_hi_c_data_f4)
      crypto_c_data_f4[127:32] <= nxt_crypto_c_data_f4[127:32];



  assign hash_data_f4 = {crypto_c_data_f4, crypto_b_data_f4};

  helios_dpu_crypto_sha1cpm u_sha1cpm_f4 (
    .crypto_op_i      (crypto_op_f4),
    .hash_data_i      (hash_data_f4[159:0]),
    .schedule_data_i  (crypto_a_data_f4[95:64]),
    .sha1cpm_res_o    (sha1cpm_res_f4)
  );

  helios_dpu_crypto_sha256h u_sha256h_f4 (
    .hash_data_i      (hash_data_f4),
    .schedule_data_i  (crypto_a_data_f4[95:64]),
    .sha256h_res_o    (sha256h_res_f4)
  );

  always_comb
    case (crypto_op_f4)
      `HELIOS_CRYPTO_OP_SHA1C,
      `HELIOS_CRYPTO_OP_SHA1P,
      `HELIOS_CRYPTO_OP_SHA1M: begin
        nxt_crypto_b_data_f5 = sha1cpm_res_f4[95:0];
        nxt_crypto_c_data_f5 = {{64{1'b0}}, sha1cpm_res_f4[159:96]};
      end


      `HELIOS_CRYPTO_OP_SHA256H: begin
        nxt_crypto_b_data_f5 = sha256h_res_f4[ 95:  0];
        nxt_crypto_c_data_f5 = sha256h_res_f4[255:128];
      end

       `HELIOS_CRYPTO_OP_SHA256H2: begin
        nxt_crypto_b_data_f5 = {sha256h_res_f4[127: 96], sha256h_res_f4[63: 0]};
        nxt_crypto_c_data_f5 = sha256h_res_f4[255:128];
      end

       `HELIOS_CRYPTO_OP_SHA256SU1: begin
        nxt_crypto_b_data_f5 = crypto_b_data_f4[95:0];
        nxt_crypto_c_data_f5 = {{96{1'b0}}, crypto_b_data_f4[127:96]};
       end

      default: begin
        nxt_crypto_b_data_f5 = '0;
        nxt_crypto_c_data_f5 = '0;
      end
    endcase

  assign en_hi_c_data_f5 = crypto_enable_f4 & advance_pipeline_i &
                           ((crypto_op_f4 == `HELIOS_CRYPTO_OP_SHA256H) |
                            (crypto_op_f4 == `HELIOS_CRYPTO_OP_SHA256H2));

  assign en_crypto_op_f4 = crypto_enable_f4 & advance_pipeline_i;

  always_ff @(posedge clk_crypto or negedge det_reset_n)
    if (~det_reset_n) begin
      crypto_a_data_f5[127:96] <= '0;
      crypto_b_data_f5         <= '0;
      crypto_c_data_f5[ 63: 0] <= '0;
      crypto_op_f5             <= '0;
    end else if (en_crypto_op_f4) begin
      crypto_a_data_f5[127:96] <= crypto_a_data_f4[127:96];
      crypto_b_data_f5         <= nxt_crypto_b_data_f5;
      crypto_c_data_f5[ 63: 0] <= nxt_crypto_c_data_f5[63: 0];
      crypto_op_f5             <= crypto_op_f4;
    end

  always_ff @(posedge clk_crypto or negedge det_reset_n)
    if (~det_reset_n)
      crypto_c_data_f5[127:64] <= '0;
    else if (en_hi_c_data_f5)
      crypto_c_data_f5[127:64] <= nxt_crypto_c_data_f5[127:64];



  assign hash_data_sha1_f5 = {crypto_c_data_f5[63:0], crypto_b_data_f5};

  helios_dpu_crypto_sha1cpm u_sha1cpm_f5 (
    .crypto_op_i      (crypto_op_f5),
    .hash_data_i      (hash_data_sha1_f5),
    .schedule_data_i  (crypto_a_data_f5[127:96]),
    .sha1cpm_res_o    (sha1cpm_res_f5)
  );

  assign hash_data_sha256_f5 = {crypto_c_data_f5, crypto_b_data_f5[95:64], crypto_b_data_f5};

  helios_dpu_crypto_sha256h u_sha256h_f5 (
    .hash_data_i      (hash_data_sha256_f5),
    .schedule_data_i  (crypto_a_data_f5[127:96]),
    .sha256h_res_o    (sha256h_res_f5)
  );

  always_comb
    case (crypto_op_f5)
      `HELIOS_CRYPTO_OP_SHA1C,
      `HELIOS_CRYPTO_OP_SHA1P,
      `HELIOS_CRYPTO_OP_SHA1M:
        crypto_data_f5_o = sha1cpm_res_f5[127:  0];

      `HELIOS_CRYPTO_OP_SHA256H:
        crypto_data_f5_o = sha256h_res_f5[127:  0];

      `HELIOS_CRYPTO_OP_SHA256H2:
        crypto_data_f5_o = sha256h_res_f5[255:128];

      `HELIOS_CRYPTO_OP_SHA256SU1:
        crypto_data_f5_o = {crypto_c_data_f5[31:0], crypto_b_data_f5};

      default:
        crypto_data_f5_o = '0;
    endcase


  assign crypto_active_o = crypto_enable_f2 | nxt_crypto_enable_f4;



endmodule

