//-----------------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited or its affiliates.
//
//            (C) COPYRIGHT 2012-2020 ARM Limited or its affiliates.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited or its affiliates.
//
//            Release Information : CORTEX-A65AE-MP083-r1p0-00eac0
//
//-----------------------------------------------------------------------------
// SystemVerilog-2012 (IEEE Std 1800-2012)
//-----------------------------------------------------------------------------


module helios_dpu_crypto_alu_sha #(parameter IS_PIPE0 = 1'b1) (
  input  logic  [63:0] fad_a_data_f2_i,
  input  logic  [63:0] fad_b_data_f2_i,
  input  logic  [63:0] fad_c_data_f2_i,
  output logic  [63:0] sha1h_res_f2_o,
  output logic  [63:0] sha1su0_opa_f2_o,
  output logic  [63:0] sha1su0_opb_f2_o,
  output logic  [63:0] sha256su0_opa_f2_o,
  output logic  [63:0] sha256su0_opb_f2_o
);


  generate if (IS_PIPE0) begin : g_sha1h_low
    assign sha1h_res_f2_o = { {32{1'b0}}, fad_a_data_f2_i[1:0], fad_a_data_f2_i[31:2]};
  end else begin : g_sha1h_high
    assign sha1h_res_f2_o = {64{1'b0}};
  end endgenerate

  assign sha1su0_opa_f2_o = fad_a_data_f2_i;
  assign sha1su0_opb_f2_o = fad_b_data_f2_i ^ fad_c_data_f2_i;

// WATERMARK START
  assign sha256su0_opa_f2_o[31: 0] = {fad_a_data_f2_i[38:32], fad_a_data_f2_i[63:39]} ^ {fad_a_data_f2_i[49:32], fad_a_data_f2_i[63:50]} ^ {3'b000, fad_a_data_f2_i[63:35]};
  assign sha256su0_opa_f2_o[63:32] = {fad_b_data_f2_i[ 6: 0], fad_b_data_f2_i[31: 7]} ^ {fad_b_data_f2_i[17: 0], fad_b_data_f2_i[31:18]} ^ {3'b000, fad_b_data_f2_i[31: 3]};
// WATERMARK VARIANTS 1
//  assign sha256su0_opa_f2_o[31: 0] = {fad_a_data_f2_i[49:32], fad_a_data_f2_i[63:50]} ^ {fad_a_data_f2_i[38:32], fad_a_data_f2_i[63:39]} ^ {3'b000, fad_a_data_f2_i[63:35]};
//  assign sha256su0_opa_f2_o[63:32] = {fad_b_data_f2_i[17: 0], fad_b_data_f2_i[31:18]} ^ {fad_b_data_f2_i[ 6: 0], fad_b_data_f2_i[31: 7]} ^ {3'b000, fad_b_data_f2_i[31: 3]};
// WATERMARK VARIANTS 2
//  assign sha256su0_opa_f2_o[31: 0] = {3'b000, fad_a_data_f2_i[63:35]} ^ {fad_a_data_f2_i[49:32], fad_a_data_f2_i[63:50]} ^ {fad_a_data_f2_i[38:32], fad_a_data_f2_i[63:39]};
//  assign sha256su0_opa_f2_o[63:32] = {3'b000, fad_b_data_f2_i[31: 3]} ^ {fad_b_data_f2_i[17: 0], fad_b_data_f2_i[31:18]} ^ {fad_b_data_f2_i[ 6: 0], fad_b_data_f2_i[31: 7]};
// WATERMARK VARIANTS 3
//  assign sha256su0_opa_f2_o[63:32] = {fad_b_data_f2_i[17: 0], fad_b_data_f2_i[31:18]} ^ {fad_b_data_f2_i[ 6: 0], fad_b_data_f2_i[31: 7]} ^ {3'b000, fad_b_data_f2_i[31: 3]};
//  assign sha256su0_opa_f2_o[31: 0] = {fad_a_data_f2_i[49:32], fad_a_data_f2_i[63:50]} ^ {fad_a_data_f2_i[38:32], fad_a_data_f2_i[63:39]} ^ {3'b000, fad_a_data_f2_i[63:35]};
// WATERMARK VARIANTS 4
//  assign sha256su0_opa_f2_o[63:32] = {3'b000, fad_b_data_f2_i[31: 3]} ^ {fad_b_data_f2_i[17: 0], fad_b_data_f2_i[31:18]} ^ {fad_b_data_f2_i[ 6: 0], fad_b_data_f2_i[31: 7]};
//  assign sha256su0_opa_f2_o[31: 0] = {3'b000, fad_a_data_f2_i[63:35]} ^ {fad_a_data_f2_i[49:32], fad_a_data_f2_i[63:50]} ^ {fad_a_data_f2_i[38:32], fad_a_data_f2_i[63:39]};
// WATERMARK END

  assign sha256su0_opb_f2_o = fad_a_data_f2_i;

endmodule

