//-----------------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited or its affiliates.
//
//            (C) COPYRIGHT 2012-2020 ARM Limited or its affiliates.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited or its affiliates.
//
//            Release Information : CORTEX-A65AE-MP083-r1p0-00eac0
//
//-----------------------------------------------------------------------------
// SystemVerilog-2012 (IEEE Std 1800-2012)
//-----------------------------------------------------------------------------


`include "helios_dpu_params.sv"
`include "helios_params.sv"

module helios_dpu_crypto_sha1cpm (
  input  logic [`HELIOS_CRYPTO_OP_W-1:0] crypto_op_i,
  input  logic                   [159:0] hash_data_i,
  input  logic                    [31:0] schedule_data_i,
  output logic                   [159:0] sha1cpm_res_o
);



  logic [31: 0] t;
  logic [31: 0] a;
  logic [31: 0] a_prime;
  logic [31: 0] b;
  logic [31: 0] b_prime;
  logic [31: 0] c;
  logic [31: 0] c_prime;
  logic [31: 0] d;
  logic [31: 0] d_prime;
  logic [31: 0] e;
  logic [31: 0] e_prime;



  assign a = hash_data_i[ 31:  0];
  assign b = hash_data_i[ 63: 32];
  assign c = hash_data_i[ 95: 64];
  assign d = hash_data_i[127: 96];
  assign e = hash_data_i[159:128];

  always_comb
    case (crypto_op_i)
      `HELIOS_CRYPTO_OP_SHA1C: t = (b & c) | (~b & d);
      `HELIOS_CRYPTO_OP_SHA1P: t = b ^ c ^ d;
      `HELIOS_CRYPTO_OP_SHA1M: t = (b & c) | (c & d) | (d & b);
      default:                 t = '0;
    endcase

  assign a_prime = e + {a[26:0], a[31:27]} + t + schedule_data_i;
  assign b_prime = a;
  assign c_prime = {b[1:0], b[31:2]};
  assign d_prime = c;
  assign e_prime = d;
  assign sha1cpm_res_o = {e_prime, d_prime, c_prime, b_prime, a_prime};

endmodule

