//-----------------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited or its affiliates.
//
//            (C) COPYRIGHT 2012-2020 ARM Limited or its affiliates.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited or its affiliates.
//
//            Release Information : CORTEX-A65AE-MP083-r1p0-00eac0
//
//-----------------------------------------------------------------------------
// SystemVerilog-2012 (IEEE Std 1800-2012)
//-----------------------------------------------------------------------------


module helios_dpu_crypto_sha256su1 (
  input  logic [63:0] op1_data_i,
  input  logic [63:0] t0_data_i,
  input  logic [63:0] t1_data_i,
  output logic [63:0] sha256su1_res_o
);


  logic [31:0] elt0_step0;
  logic [31:0] elt0_step1;
  logic [31:0] elt1_step0;
  logic [31:0] elt1_step1;


  assign elt0_step0 = t1_data_i[31: 0];
  assign elt1_step0 = t1_data_i[63:32];

// WATERMARK START
  assign elt0_step1 = {elt0_step0[16:0], elt0_step0[31:17]} ^ {elt0_step0[18:0], elt0_step0[31:19]} ^ {10'h000, elt0_step0[31:10]};
  assign elt1_step1 = {elt1_step0[16:0], elt1_step0[31:17]} ^ {elt1_step0[18:0], elt1_step0[31:19]} ^ {10'h000, elt1_step0[31:10]};
// WATERMARK VARIANTS 1
//  assign elt0_step1 = {elt0_step0[18:0], elt0_step0[31:19]} ^ {elt0_step0[16:0], elt0_step0[31:17]} ^ {10'h000, elt0_step0[31:10]};
//  assign elt1_step1 = {elt1_step0[18:0], elt1_step0[31:19]} ^ {elt1_step0[16:0], elt1_step0[31:17]} ^ {10'h000, elt1_step0[31:10]};
// WATERMARK VARIANTS 2
//  assign elt0_step1 = {10'h000, elt0_step0[31:10]} ^ {elt0_step0[18:0], elt0_step0[31:19]} ^ {elt0_step0[16:0], elt0_step0[31:17]};
//  assign elt1_step1 = {10'h000, elt1_step0[31:10]} ^ {elt1_step0[18:0], elt1_step0[31:19]} ^ {elt1_step0[16:0], elt1_step0[31:17]};
// WATERMARK VARIANTS 3
//  assign elt1_step1 = {elt1_step0[18:0], elt1_step0[31:19]} ^ {elt1_step0[16:0], elt1_step0[31:17]} ^ {10'h000, elt1_step0[31:10]};
//  assign elt0_step1 = {elt0_step0[18:0], elt0_step0[31:19]} ^ {elt0_step0[16:0], elt0_step0[31:17]} ^ {10'h000, elt0_step0[31:10]};
// WATERMARK VARIANTS 4
//  assign elt1_step1 = {10'h000, elt1_step0[31:10]} ^ {elt1_step0[18:0], elt1_step0[31:19]} ^ {elt1_step0[16:0], elt1_step0[31:17]};
//  assign elt0_step1 = {10'h000, elt0_step0[31:10]} ^ {elt0_step0[18:0], elt0_step0[31:19]} ^ {elt0_step0[16:0], elt0_step0[31:17]};
// WATERMARK END

// WATERMARK START
  assign sha256su1_res_o[31: 0] = elt0_step1 + op1_data_i[31: 0] + t0_data_i[31: 0];
  assign sha256su1_res_o[63:32] = elt1_step1 + op1_data_i[63:32] + t0_data_i[63:32];
// WATERMARK VARIANTS 1
//  assign sha256su1_res_o[31: 0] = elt0_step1 + t0_data_i[31: 0] + op1_data_i[31: 0];
//  assign sha256su1_res_o[63:32] = elt1_step1 + t0_data_i[63:32] + op1_data_i[63:32];
// WATERMARK VARIANTS 2
//  assign sha256su1_res_o[31: 0] = t0_data_i[31: 0] + op1_data_i[31: 0] + elt0_step1;
//  assign sha256su1_res_o[63:32] = t0_data_i[63:32] + op1_data_i[63:32] + elt1_step1;
// WATERMARK VARIANTS 3
//  assign sha256su1_res_o[63:32] = elt1_step1 + t0_data_i[63:32] + op1_data_i[63:32];
//  assign sha256su1_res_o[31: 0] = elt0_step1 + t0_data_i[31: 0] + op1_data_i[31: 0];
// WATERMARK VARIANTS 4
//  assign sha256su1_res_o[63:32] = t0_data_i[63:32] + op1_data_i[63:32] + elt1_step1;
//  assign sha256su1_res_o[31: 0] = t0_data_i[31: 0] + op1_data_i[31: 0] + elt0_step1;
// WATERMARK END

endmodule

